﻿' This software is distributed under the license of NYSL.
' ( http://www.kmonos.net/nysl/ )
Imports System.ComponentModel

''' <summary>
'''  ラジオボタンセル(DataGridViewRadioButtonCell)のホスト
''' </summary>
Public Class DataGridViewRadioButtonColumn
    Inherits DataGridViewColumn

    ''' <summary>ラジオボタンが属するグループ名</summary>
    Private _groupName As String

    ''' <summary>
    ''' 
    ''' </summary>
    Public Sub New()
        MyBase.New(New DataGridViewRadioButtonCell)
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <returns></returns>
    Public Overrides Function Clone() As Object

        Dim col As DataGridViewRadioButtonColumn = _
            DirectCast(MyBase.Clone(), DataGridViewRadioButtonColumn)

        ' プロパティを追加したとき、この場所に追記する
        ' Cloneメソッドに記述しない場合、プロパティブラウザで表示・編集は
        ' されるが、保存されないプロパティとなる
        col.GroupName = Me.GroupName

        Return col
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    Public Overrides Property CellTemplate() As DataGridViewCell
        Get
            Return MyBase.CellTemplate
        End Get
        Set(ByVal value As DataGridViewCell)
            If Not (value Is Nothing) AndAlso _
                Not value.GetType().IsAssignableFrom(GetType(DataGridViewRadioButtonCell)) Then
                Throw New InvalidCastException("Must be a DataGridViewRadioButtonCell")
            End If
            MyBase.CellTemplate = value
        End Set
    End Property

    ''' <summary>
    ''' ラジオボタンが属するグループ名
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' ここで設定されたグループ名により、同一行内・同一グループのチェック
    ''' 状態のラジオボタンがひとつになるよう制御される</remarks>
    <Browsable(True), DefaultValue(GetType(System.String), "defaultGroup")>
    Public Property GroupName As String
        Get
            Return _groupName
        End Get
        Set(value As String)
            _groupName = value
        End Set
    End Property
End Class

''' <summary>
''' ラジオボタンのセルクラス
''' </summary>
''' <remarks></remarks>
Public Class DataGridViewRadioButtonCell
    Inherits DataGridViewCell

    ''' <summary>ラジオボタンの描画サイズ</summary>
    Private Const RADIO_SIZE As Integer = 16

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()
    End Sub

    ''' <summary>
    ''' 1
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides Function Clone() As Object
        Dim cell = DirectCast(MyBase.Clone(), DataGridViewRadioButtonCell)
        Return cell
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="value"></param>
    ''' <param name="rowIndex"></param>
    ''' <param name="cellStyle"></param>
    ''' <param name="valueTypeConverter"></param>
    ''' <param name="formattedValueTypeConverter"></param>
    ''' <param name="context"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Protected Overrides Function GetFormattedValue(value As Object, rowIndex As Integer, ByRef cellStyle As DataGridViewCellStyle, valueTypeConverter As TypeConverter, formattedValueTypeConverter As TypeConverter, context As DataGridViewDataErrorContexts) As Object
        Return _val(value)
    End Function

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="v"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Shared Function _val(v As Object) As Boolean
        Try
            Return If(IsDBNull(v) OrElse IsNothing(v), False, Convert.ToBoolean(v))
        Catch ex As Exception
            Return False
        End Try
    End Function
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="graphics"></param>
    ''' <param name="clipBounds"></param>
    ''' <param name="cellBounds"></param>
    ''' <param name="rowIndex"></param>
    ''' <param name="elementState"></param>
    ''' <param name="value"></param>
    ''' <param name="formattedValue"></param>
    ''' <param name="errorText"></param>
    ''' <param name="cellStyle"></param>
    ''' <param name="advancedBorderStyle"></param>
    ''' <param name="paintParts"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub Paint(graphics As System.Drawing.Graphics, _
                                  clipBounds As System.Drawing.Rectangle, _
                                  cellBounds As System.Drawing.Rectangle, _
                                  rowIndex As Integer, _
                                  elementState As System.Windows.Forms.DataGridViewElementStates, _
                                  value As Object, _
                                  formattedValue As Object, _
                                  errorText As String, _
                                  cellStyle As System.Windows.Forms.DataGridViewCellStyle, _
                                  advancedBorderStyle As System.Windows.Forms.DataGridViewAdvancedBorderStyle, _
                                  paintParts As System.Windows.Forms.DataGridViewPaintParts)

        MyBase.Paint(graphics, clipBounds, cellBounds, rowIndex, elementState, value, formattedValue, errorText, cellStyle, advancedBorderStyle, paintParts)

        ' 背景色の描画
        If (paintParts And DataGridViewPaintParts.Background) = DataGridViewPaintParts.Background Then
            Dim brush As New SolidBrush(If(elementState And DataGridViewElementStates.Selected, _
                                           cellStyle.SelectionBackColor, cellStyle.BackColor))
            graphics.FillRectangle(brush, cellBounds)
            brush.Dispose()
        End If

        ' 境界線の描画
        If (paintParts And DataGridViewPaintParts.Border) = DataGridViewPaintParts.Border Then
            PaintBorder(graphics, clipBounds, cellBounds, cellStyle, advancedBorderStyle)
        End If

        ' セルの幅、高さがラジオボタンの描画サイズより小さい場合は、描画処理を飛ばす
        If cellBounds.Width < RADIO_SIZE OrElse cellBounds.Height < RADIO_SIZE Then
            Return
        End If

        ' ラジオボタンをセルの真ん中に描画(描画サイズは固定)
        Dim rect As Rectangle = New Rectangle(New Point(cellBounds.Left + (cellBounds.Width - RADIO_SIZE) / 2, _
                                                        cellBounds.Top + (cellBounds.Height - RADIO_SIZE) / 2), _
                                                        New Size(RADIO_SIZE, RADIO_SIZE))


        Dim v = _val(value)
        ControlPaint.DrawRadioButton(graphics, rect, If(v, ButtonState.Checked, ButtonState.Inactive))

    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="e"></param>
    ''' <param name="rowIndex"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnKeyPress(e As System.Windows.Forms.KeyPressEventArgs, rowIndex As Integer)

        MyBase.OnKeyPress(e, rowIndex)

        If [ReadOnly] Then
            Return
        End If

        ' スペースキーが押されたとき、ラジオボタンのチェック状態の更新
        If e.KeyChar = Microsoft.VisualBasic.ChrW(Keys.Space) Then
            updateValues(DataGridView.CurrentCell.ColumnIndex, rowIndex)
        End If

    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnClick(e As System.Windows.Forms.DataGridViewCellEventArgs)

        MyBase.OnClick(e)

        If [ReadOnly] Then
            Return
        End If

        ' ラジオボタンのチェック状態の更新
        updateValues(e.ColumnIndex, e.RowIndex)

    End Sub

    ''' <summary>
    ''' ラジオボタンのチェック状態を更新
    ''' </summary>
    ''' <param name="columnIndex"></param>
    ''' <param name="rowIndex"></param>
    ''' <remarks></remarks>
    Private Sub updateValues(columnIndex As Integer, rowIndex As Integer)

        Dim currentColumn As DataGridViewRadioButtonColumn = _
            TryCast(Me.DataGridView.Columns(columnIndex), DataGridViewRadioButtonColumn)

        For Each column As DataGridViewColumn In Me.DataGridView.Columns

            ' ラジオボタンセル以外は処理を飛ばす
            If Not (TypeOf (column) Is DataGridViewRadioButtonColumn) Then
                Continue For
            End If

            ' カレントセルと異なるグループ名は処理を飛ばす
            Dim radioColumn As DataGridViewRadioButtonColumn = TryCast(column, DataGridViewRadioButtonColumn)
            If radioColumn.GroupName <> currentColumn.GroupName Then
                Continue For
            End If

            Dim dgvc = Me.DataGridView.Rows(rowIndex).Cells(column.Index)

            ' 更新後のチェック状態を求める
            Dim v = If(radioColumn.Index = currentColumn.Index, True, False)

            ' 値の更新が生じない場合は処理を飛ばす
            Dim prev = _val(dgvc.Value)
            If prev = v Then
                Continue For
            End If

            dgvc.Value = v

            ' イベントを発生させる
            ' ※チェック状態を更新した回数分イベントが発生する
            Dim newEa = New DataGridViewCellEventArgs(columnIndex, rowIndex)
            RaiseCellValueChanged(newEa)

            Me.DataGridView.InvalidateCell(dgvc)
            Me.DataGridView.NotifyCurrentCellDirty(True)
        Next

    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides ReadOnly Property DefaultNewRowValue As Object
        Get
            Return False
        End Get
    End Property

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides Property ValueType As System.Type
        Get
            Return GetType(Boolean)
        End Get
        Set(value As System.Type)
            Throw New InvalidOperationException
        End Set
    End Property

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides ReadOnly Property FormattedValueType As Type
        Get
            Return GetType(Boolean)
        End Get
    End Property

End Class
